SystemOrganization addCategory: #'XMLRPC-Server'!
SystemOrganization addCategory: 'XMLRPC-Server-Errors'!
SystemOrganization addCategory: 'XMLRPC-Server-Test'!
SystemOrganization addCategory: 'XMLRPC-Server-Validation'!

Object subclass: #XMLRPCErrorHandler
	instanceVariableNames: 'error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

!XMLRPCErrorHandler commentStamp: '<historical>' prior: 0!
This is the error handler for XMLRPC errors!

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: ''!
error
	"Answer the receiver's 'error'."

	^error! !

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: ''!
error: anObject
	"Set the receiver's instance variable 'error' to be anObject."

	error := anObject! !

!XMLRPCErrorHandler methodsFor: 'accessing' stamp: 'SantiagoBragagnolo 9/14/2012 11:54'!
errorMessage
	| str errorDict |
	errorDict := Dictionary new.
	
	(error class canUnderstand: #faultCode) ifTrue: [
		errorDict at: #faultCode put: error faultCode
	] ifFalse: [
		errorDict at: #faultCode put: 10
	]. 

	errorDict at: #faultString put: error messageText.
	
	str := WriteStream on: (String new: 1000).
	
	str 	nextPutAll: '<?xml version="1.0"?><methodResponse><fault>';
		nextPutAll: errorDict asXMLRPCString;
		nextPutAll: '</fault></methodResponse>'.
		
	^ str contents! !

Object subclass: #XMLRPCHttpModule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

XMLRPCHttpModule class
	instanceVariableNames: 'port service'!

!XMLRPCHttpModule commentStamp: '<historical>' prior: 0!
This comanche module handles incoming requests for a XMLRPC server!

XMLRPCHttpModule class
	instanceVariableNames: 'port service'!

!XMLRPCHttpModule class methodsFor: 'construction' stamp: 'SantiagoBragagnolo 9/20/2012 11:00'!
for: aServiceDefinition 
	^ self new services: aServiceDefinition ; yourself.! !

!XMLRPCHttpModule methodsFor: 'logging' stamp: 'SantiagoBragagnolo 9/19/2012 17:17'!
initialize 
	super initialize.
	services := Dictionary new.
	! !

!XMLRPCHttpModule methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/27/2012 16:58'!
invoke: aService with: anArrayOfParameters
	| association target selector |
	association := (services at: aService).
	target := association first key.
	selector := association first value.

	[
		^ target perform: selector withArguments: anArrayOfParameters asArray.
	] on: Error do: [
		:e |
		XMLRPCMethodError new signal: 'Message not understood by server class: ', e messageText.
	]! !

!XMLRPCHttpModule methodsFor: 'process' stamp: 'LucFabresse 2/1/2015 23:07'!
process: request 
	| xmldoc  parameters service response  |
	
	[
		xmldoc := XMLDOMParser parse: (request contents) readStream.		
		service := self resolveServiceName: xmldoc.
		parameters := self resolveParameters: xmldoc.
		response :=  (service = 'system.multicall') ifTrue: [
			(parameters at:1) collect: [ : i | 
				| s p |
				s :=  i at:'methodName'  .
				p := i at: 'params'.
				self invoke: s with: p. 	
			]
		] ifFalse: [ 
			self invoke: service with: parameters . 	
		 ].
		response := self returnableResponse: response.
		
	] on: Error do:[ 
		:e |
		 response := (XMLRPCErrorHandler new error: e) errorMessage.
	].
	^ response.! !

!XMLRPCHttpModule methodsFor: 'registering' stamp: 'SantiagoBragagnolo 9/19/2012 17:16'!
registerService: serviceName target: aTarget selector: aSelector 
	services at: serviceName put: { aTarget -> aSelector asSymbol }.! !

!XMLRPCHttpModule methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/19/2012 17:28'!
resolveParameters: xmldoc
	^((xmldoc firstElement) elementAt: #params ) elements inject: OrderedCollection new into: [
		:parameters :parameter |
		parameters add: (XMLRPCDecoder new decode: parameter); yourself.
	].! !

!XMLRPCHttpModule methodsFor: 'private' stamp: 'SantiagoBragagnolo 9/19/2012 17:28'!
resolveServiceName: xmldoc
	^ ((xmldoc firstElement elementAt: #methodName) contentString).	! !

!XMLRPCHttpModule methodsFor: 'private' stamp: 'SantiagoBragagnolo 1/25/2013 10:55'!
returnableResponse: aResult
	| stream |
	stream := WriteStream on: String new.
	stream nextPutAll: '<?xml version="1.0"?><methodResponse><params><param>'.
	stream nextPutAll: aResult asXMLRPCString.
	stream nextPutAll: '</param></params></methodResponse>'.
	
	
	^ stream contents.
	! !

!XMLRPCHttpModule methodsFor: 'accessing' stamp: 'SantiagoBragagnolo 9/19/2012 17:42'!
services: aServiceDefinition
	services := aServiceDefinition asDictionary.! !

Object subclass: #XMLRPCServerRequest
	instanceVariableNames: 'methodCall params targetURI'
	classVariableNames: 'Receivers'
	poolDictionaries: ''
	category: 'XMLRPC-Server'!

!XMLRPCServerRequest commentStamp: '<historical>' prior: 0!
This class holds all information about a XMLRPC request an coordinates the computation of a XMLRPC methodcall!

!XMLRPCServerRequest class methodsFor: 'as yet unclassified' stamp: 'SantiagoBragagnolo 9/17/2012 15:57'!
cleanUpReceiverDictionary
	" deletes references to obsolete classes from the 'Receivers' dictionary"
	| removeThem |
	
	self debug: ['Cleaning up receiving dictionary: ' , self receivers ].
	removeThem := self receivers 	select: [ : each | each isObsolete ].
	removeThem keysDo: [ : key | self receivers removeKey: key ].
	! !

!XMLRPCServerRequest class methodsFor: 'as yet unclassified' stamp: 'LucFabresse 2/2/2015 08:20'!
fromRequest: xmlString 
	| xmldoc r parameters relatedSelector |
		
	r := self new.
	parameters := OrderedCollection new.
	xmldoc := XMLDOMParser parseDocumentFrom: xmlString readStream.
	
	relatedSelector := ((xmldoc firstElement elementAt: #methodName) contentString).
	
	r methodCall: relatedSelector.	
	[
		((xmldoc firstElement) elementAt: #params ) elements do: [
			:each | 
			parameters add: (XMLRPCDecoder new decode: each)
		].
		
	] on: Error do: [
		XMLRPCRequestError new signal: 'Could not decode request'
	].

	r params: parameters.
	^ r! !

!XMLRPCServerRequest class methodsFor: 'class initialization' stamp: ''!
initialize
	Receivers := Dictionary new.! !

XMLRPCServerRequest initialize!

!XMLRPCServerRequest class methodsFor: 'accessing' stamp: 'SantiagoBragagnolo 9/14/2012 11:56'!
receivers
	^Receivers ifNil: [Receivers := Dictionary new].! !

!XMLRPCServerRequest class methodsFor: 'as yet unclassified' stamp: 'SantiagoBragagnolo 1/25/2013 12:15'!
registerService: serviceName class: aClass selector: aSelector 
	self receivers at: serviceName asSymbol put: aClass -> aSelector! !

!XMLRPCServerRequest class methodsFor: 'as yet unclassified' stamp: 'LucFabresse 2/2/2015 08:19'!
removeAllServices
	self receivers removeAll.! !

!XMLRPCServerRequest class methodsFor: 'as yet unclassified' stamp: 'LucFabresse 2/2/2015 08:20'!
removeService: aXMLRPCService
	self receivers removeKey: aXMLRPCService asSymbol! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
methodCall
	"Answer the receiver's 'methodCall'."

	^methodCall! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
methodCall: anObject
	"Set the receiver's instance variable 'methodCall' to be anObject."

	methodCall := anObject! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
params
	"Answer the receiver's 'params'."

	^params! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
params: anObject
	"Set the receiver's instance variable 'params' to be anObject."

	params := anObject! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
targetURI
	"Answer the receiver's 'targetURI'."

	^targetURI! !

!XMLRPCServerRequest methodsFor: 'accessing' stamp: ''!
targetURI: anObject
	"Set the receiver's instance variable 'targetURI' to be anObject."

	targetURI := anObject! !

Object subclass: #XMLRPCStandardValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Validation'!

Error subclass: #XMLRPCError
	instanceVariableNames: 'faultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCError commentStamp: '<historical>' prior: 0!
This is a generic error, it is raised if an unknown error has been detected.!

XMLRPCError subclass: #XMLRPCClassNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCClassNotFoundError commentStamp: '<historical>' prior: 0!
The request Class wasn't found!

!XMLRPCClassNotFoundError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 1! !

!XMLRPCError methodsFor: 'accessing' stamp: ''!
faultCode
	"Answer the receiver's 'faultCode'."
	^ faultCode ifNil: [ ^0]! !

!XMLRPCError methodsFor: 'accessing' stamp: ''!
faultCode: anObject
	"Set the receiver's instance variable 'faultCode' to be anObject."

	faultCode := anObject! !

XMLRPCError subclass: #XMLRPCMethodError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCMethodError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 2! !

XMLRPCError subclass: #XMLRPCRequestError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCRequestError commentStamp: '<historical>' prior: 0!
An error raised if a malformed request was found!

!XMLRPCRequestError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 3! !

XMLRPCError subclass: #XMLRPCReturnError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Errors'!

!XMLRPCReturnError commentStamp: '<historical>' prior: 0!
An error occured while encoding a methodresponse!

!XMLRPCReturnError methodsFor: 'as yet unclassified' stamp: ''!
faultCode
	^ 4! !

TestCase subclass: #XMLRPCServerTest
	instanceVariableNames: 'newServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XMLRPC-Server-Test'!

!XMLRPCServerTest methodsFor: 'running' stamp: 'LucFabresse 5/8/2015 18:31'!
setUp
	server := XMLRPCServerForTest new startOn: 8081! !

!XMLRPCServerTest methodsFor: 'running' stamp: 'LucFabresse 5/8/2015 18:31'!
tearDown
	server stop! !

!XMLRPCServerTest methodsFor: 'tests' stamp: 'LucFabresse 5/8/2015 18:32'!
testDictionary
	| d p r |
	d := Dictionary new.
	d at: 'true' put: true.
	d at: 'Markus' put: 'Fritsche'.
	p := XMLRPCProxy new url: 'http://localhost:8081/' asUrl.
	r := p invokeMethod: 'validator1.echoStructTest' withStruct: d.
	self assert: r = d! !

!XMLRPCServerTest methodsFor: 'tests' stamp: 'LucFabresse 5/8/2015 18:32'!
testHallo
	| p |
	p := XMLRPCProxy new url: 'http://localhost:8081/' asUrl.
	p invokeMethod: 'validator1.echoStructTest' withArgs: #('Hallo')
! !

!XMLRPCServerTest methodsFor: 'tests' stamp: 'LucFabresse 5/8/2015 18:43'!
testThroughClient
" bring in tests here independently in next version.. for now just reuse the XMLRPCTests"

 | client |
 client := XMLRPCClientTest suite.
 client run.
! !

XMLRPCServerRequest initialize!
